@@ -30,9 +30,9 @@ SOFTWARE.
 
 
 # Set version information.
-__version_info__ = ('0', '1', '5')
+__version_info__ = ('0', '1', '7')
 __version__ = '.'.join(__version_info__)
 
 
 # Import local modules.
-from avro.main import *
+from .main import *
